searchData={"items":[{"type":"module","title":"Cloak","doc":"Cloak consists of two behaviours:\n\n- `Cloak.Cipher` - Ciphers encrypt and decrypt data using a particular\n  encryption algorithm and key.\n\n- `Cloak.Vault` - Vaults configure and use ciphers. Applications can\n  have multiple vaults.","ref":"Cloak.html"},{"type":"behaviour","title":"Cloak.Cipher","doc":"A behaviour for encryption/decryption modules. You can rely on this behaviour\nto create your own Cloak-compatible cipher modules.","ref":"Cloak.Cipher.html"},{"type":"behaviour","title":"Example - Cloak.Cipher","doc":"We will create a cipher that simply prepends `\"Hello, \"` to any given\nplaintext on encryption, and removes the prefix on decryption.\n\nFirst, define your own cipher module, and specify the `Cloak.Cipher`\nbehaviour.\n\n    defmodule MyApp.PrefixCipher do\n      @behaviour Cloak.Cipher\n    end\n\nAdd some configuration to your vault for this new cipher:\n\n    config :my_app, MyApp.Vault,\n      ciphers: [\n        prefix: {MyApp.PrefixCipher, prefix: \"Hello, \"}\n      ]\n\nThe keyword list containing the `:prefix` will be passed in as `opts`\nto our cipher callbacks. You should specify any options your cipher will\nneed for encryption/decryption here, such as the key.\n\nNext, define the `can_decrypt?/2` callback:\n\n    @impl true\n    def can_decrypt?(ciphertext, opts) do\n      String.starts_with?(ciphertext, opts[:prefix])\n    end\n\nIf the ciphertext starts with `\"Hello, \"`, we know it was encrypted with this\ncipher and we can proceed. Finally, define the `encrypt` and `decrypt`\nfunctions:\n\n    @impl true\n    def encrypt(plaintext, opts) do\n      opts[:prefix] <> plaintext\n    end\n\n    @impl true\n    def decrypt(ciphertext, opts) do\n      String.replace(ciphertext, opts[:prefix], \"\")\n    end\n\nYou can now use your cipher with your vault!\n\n    MyApp.Vault.encrypt!(\"World!\", :prefix)\n    # => \"Hello, World!\"\n\n    MyApp.Vault.decrypt!(\"Hello, World!\")\n    # => \"World!\"","ref":"Cloak.Cipher.html#module-example"},{"type":"callback","title":"Cloak.Cipher.can_decrypt?/2","doc":"Determines if a given ciphertext can be decrypted by this cipher. Options\nare derived from the cipher configuration. See `encrypt/2`.","ref":"Cloak.Cipher.html#c:can_decrypt?/2"},{"type":"callback","title":"Cloak.Cipher.decrypt/2","doc":"Decrypt a value, using the given opts. Options are derived from the cipher\nconfiguration. See `encrypt/2`.","ref":"Cloak.Cipher.html#c:decrypt/2"},{"type":"callback","title":"Cloak.Cipher.encrypt/2","doc":"Encrypt a value, using the given keyword list of options. These options\nderive from the cipher configuration, like so:\n\n    config :my_app, MyApp.Vault,\n      ciphers: [\n        default: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V1\", key: <<1, 0, ...>>}\n      ]\n\nThe above configuration will result in the following `opts` being passed\nto this function:\n\n    [tag: \"AES.GCM.V1\", key: <<1, 0, ...>>]\n\nYour implementation **must** include any information it will need for\ndecryption in the generated ciphertext.","ref":"Cloak.Cipher.html#c:encrypt/2"},{"type":"type","title":"Cloak.Cipher.ciphertext/0","doc":"","ref":"Cloak.Cipher.html#t:ciphertext/0"},{"type":"type","title":"Cloak.Cipher.opts/0","doc":"","ref":"Cloak.Cipher.html#t:opts/0"},{"type":"type","title":"Cloak.Cipher.plaintext/0","doc":"","ref":"Cloak.Cipher.html#t:plaintext/0"},{"type":"behaviour","title":"Cloak.Vault","doc":"Encrypts and decrypts data, using a configured cipher.","ref":"Cloak.Vault.html"},{"type":"behaviour","title":"Create Your Vault - Cloak.Vault","doc":"Define a module in your application that uses `Cloak.Vault`.\n\n    defmodule MyApp.Vault do\n      use Cloak.Vault, otp_app: :my_app\n    end","ref":"Cloak.Vault.html#module-create-your-vault"},{"type":"behaviour","title":"Configuration - Cloak.Vault","doc":"The `:otp_app` option should point to an OTP application that has the vault\nconfiguration.\n\nFor example, the vault:\n\n    defmodule MyApp.Vault do\n      use Cloak.Vault, otp_app: :my_app\n    end\n\nCould be configured with Mix configuration like so:\n\n    config :my_app, MyApp.Vault,\n      json_library: Jason,\n      ciphers: [\n        default: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V1\", key: <<...>>}\n      ]\n\nThe configuration options are:\n\n- `:json_library`: Used to convert data types like lists and maps into\n  binary so that they can be encrypted. (Default: `Jason`)\n\n- :ciphers: a list of `Cloak.Cipher` modules the following format:\n\n        {:label, {CipherModule, opts}}\n\n  **The first configured cipher in the list is the default for encrypting\n  all new data, regardless of its label.** This behaviour can be overridden\n  on a field-by-field basis.\n\n  The `opts` are specific to each cipher module. Check their\n  codumentation for what each cipher requires.\n\n    - `Cloak.Ciphers.AES.GCM`\n    - `Cloak.Ciphers.AES.CTR`\n\n#","ref":"Cloak.Vault.html#module-configuration"},{"type":"behaviour","title":"Runtime Configuration - Cloak.Vault","doc":"Because Vaults are GenServers, they can be configured at runtime using the\n`init/1` callback. This allows you to easily fetch values like environment\nvariables in a reliable way.\n\nThe configuration from the `:otp_app` is passed as the first argument to the\ncallback, allowing you to append to or change it at will.\n\n    defmodule MyApp.Vault do\n      use Cloak.Vault, otp_app: :my_app\n\n      @impl GenServer\n      def init(config) do\n        config =\n          Keyword.put(config, :ciphers, [\n            default: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V1\", key: decode_env!(\"CLOAK_KEY\")}\n          ])\n\n        {:ok, config}\n      end\n\n      defp decode_env!(var) do\n        var\n        |> System.get_env()\n        |> Base.decode64!()\n      end\n    end\n\nYou can also pass configuration to vaults via `start_link/1`:\n\n    MyApp.Vault.start_link(ciphers: [\n      default: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V1\", key: key}\n    ])","ref":"Cloak.Vault.html#module-runtime-configuration"},{"type":"behaviour","title":"Supervision - Cloak.Vault","doc":"Because Vaults are `GenServer`s, you'll need to add your vault to your\nsupervision tree in `application.ex` or whichever supervisor you prefer.\n\n    children = [\n      MyApp.Vault\n    ]\n\nIf you want to pass in configuration values at runtime, you can do so:\n\n    children = [\n      {MyApp.Vault, ciphers: [...]}\n    ]","ref":"Cloak.Vault.html#module-supervision"},{"type":"behaviour","title":"Usage - Cloak.Vault","doc":"You can use the vault directly by calling its functions.\n\n    MyApp.Vault.encrypt(\"plaintext\")\n    # => {:ok, <<...>>}\n\n    MyApp.Vault.decrypt(ciphertext)\n    # => {:ok, \"plaintext\"}\n\nSee the documented callbacks below for the functions you can call.\n\n#","ref":"Cloak.Vault.html#module-usage"},{"type":"behaviour","title":"Performance Notes - Cloak.Vault","doc":"Vaults are not bottlenecks. They simply store configuration in an ETS table\nnamed after the Vault, e.g. `MyApp.Vault.Config`. All encryption and\ndecryption is performed in your local process, reading configuration from\nthe vault's ETS table.","ref":"Cloak.Vault.html#module-performance-notes"},{"type":"callback","title":"Cloak.Vault.decrypt/1","doc":"Decrypts a binary with the configured cipher that generated the binary.\nAutomatically detects which cipher to use, based on the ciphertext.","ref":"Cloak.Vault.html#c:decrypt/1"},{"type":"callback","title":"Cloak.Vault.decrypt!/1","doc":"Like `decrypt/1`, but raises any errors.","ref":"Cloak.Vault.html#c:decrypt!/1"},{"type":"callback","title":"Cloak.Vault.encrypt/1","doc":"Encrypts a binary using the first configured cipher in the vault's\nconfigured `:ciphers` list.","ref":"Cloak.Vault.html#c:encrypt/1"},{"type":"callback","title":"Cloak.Vault.encrypt/2","doc":"Encrypts a binary using the vault's configured cipher with the\ncorresponding label.","ref":"Cloak.Vault.html#c:encrypt/2"},{"type":"callback","title":"Cloak.Vault.encrypt!/1","doc":"Like `encrypt/1`, but raises any errors.","ref":"Cloak.Vault.html#c:encrypt!/1"},{"type":"callback","title":"Cloak.Vault.encrypt!/2","doc":"Like `encrypt/2`, but raises any errors.","ref":"Cloak.Vault.html#c:encrypt!/2"},{"type":"callback","title":"Cloak.Vault.json_library/0","doc":"The JSON library the vault uses to convert maps and lists into\nJSON binaries before encryption.","ref":"Cloak.Vault.html#c:json_library/0"},{"type":"type","title":"Cloak.Vault.ciphertext/0","doc":"","ref":"Cloak.Vault.html#t:ciphertext/0"},{"type":"type","title":"Cloak.Vault.label/0","doc":"","ref":"Cloak.Vault.html#t:label/0"},{"type":"type","title":"Cloak.Vault.plaintext/0","doc":"","ref":"Cloak.Vault.html#t:plaintext/0"},{"type":"module","title":"Cloak.Ciphers.AES.CTR","doc":"A `Cloak.Cipher` which encrypts values with the AES cipher in CTR (stream) mode.\nInternally relies on Erlang's `:crypto.stream_encrypt/2`.","ref":"Cloak.Ciphers.AES.CTR.html"},{"type":"function","title":"Cloak.Ciphers.AES.CTR.can_decrypt?/2","doc":"Callback implementation for `Cloak.Cipher`. Determines if\na ciphertext can be decrypted with this cipher.","ref":"Cloak.Ciphers.AES.CTR.html#can_decrypt?/2"},{"type":"function","title":"Cloak.Ciphers.AES.CTR.decrypt/2","doc":"Callback implementation for `Cloak.Cipher`. Decrypts a value\nencrypted with AES in CTR mode.\n\nUses the key tag to find the correct key for decryption, and the IV included\nin the header to decrypt the body of the ciphertext.\n\n#","ref":"Cloak.Ciphers.AES.CTR.html#decrypt/2"},{"type":"function","title":"Parameters - Cloak.Ciphers.AES.CTR.decrypt/2","doc":"- `ciphertext` - Binary ciphertext generated by `encrypt/2`.\n\n#","ref":"Cloak.Ciphers.AES.CTR.html#decrypt/2-parameters"},{"type":"function","title":"Examples - Cloak.Ciphers.AES.CTR.decrypt/2","doc":"iex> encrypt(\"Hello\") |> decrypt\n    \"Hello\"","ref":"Cloak.Ciphers.AES.CTR.html#decrypt/2-examples"},{"type":"function","title":"Cloak.Ciphers.AES.CTR.encrypt/2","doc":"Callback implementation for `Cloak.Cipher`. Encrypts a value using\nAES in CTR mode.\n\nGenerates a random IV for every encryption, and prepends the key tag and IV to\nthe beginning of the ciphertext. The format can be diagrammed like this:\n\n    +-----------------------------------+----------------------+\n    |               HEADER              |         BODY         |\n    +-------------------+---------------+----------------------+\n    | Key Tag (n bytes) | IV (16 bytes) | Ciphertext (n bytes) |\n    +-------------------+---------------+----------------------+\n    |                   |__________________________________\n    |                                                     |\n    +---------------+-----------------+-------------------+\n    | Type (1 byte) | Length (1 byte) | Key Tag (n bytes) |\n    +---------------+-----------------+-------------------+\n\nThe `Key Tag` component of the header breaks down into a `Type`, `Length`,\nand `Value` triplet for easy decoding.","ref":"Cloak.Ciphers.AES.CTR.html#encrypt/2"},{"type":"module","title":"Cloak.Ciphers.AES.GCM","doc":"A `Cloak.Cipher` which encrypts values with the AES cipher in GCM (block) mode.\nInternally relies on Erlang's `:crypto.block_encrypt/4`.","ref":"Cloak.Ciphers.AES.GCM.html"},{"type":"function","title":"Cloak.Ciphers.AES.GCM.can_decrypt?/2","doc":"Callback implementation for `Cloak.Cipher`. Determines whether this module\ncan decrypt the given ciphertext.","ref":"Cloak.Ciphers.AES.GCM.html#can_decrypt?/2"},{"type":"function","title":"Cloak.Ciphers.AES.GCM.decrypt/2","doc":"Callback implementation for `Cloak.Cipher`. Decrypts a value\nencrypted with AES in GCM mode.","ref":"Cloak.Ciphers.AES.GCM.html#decrypt/2"},{"type":"function","title":"Cloak.Ciphers.AES.GCM.encrypt/2","doc":"Callback implementation for `Cloak.Cipher`. Encrypts a value using\nAES in GCM mode.\n\nGenerates a random IV for every encryption, and prepends the key tag, IV,\nand ciphertag to the beginning of the ciphertext. The format can be\ndiagrammed like this:\n\n    +----------------------------------------------------------+----------------------+\n    |                          HEADER                          |         BODY         |\n    +-------------------+---------------+----------------------+----------------------+\n    | Key Tag (n bytes) | IV (n bytes)  | Ciphertag (16 bytes) | Ciphertext (n bytes) |\n    +-------------------+---------------+----------------------+----------------------+\n    |                   |_________________________________\n    |                                                     |\n    +---------------+-----------------+-------------------+\n    | Type (1 byte) | Length (1 byte) | Key Tag (n bytes) |\n    +---------------+-----------------+-------------------+\n\nThe `Key Tag` component of the header breaks down into a `Type`, `Length`,\nand `Value` triplet for easy decoding.\n\n**Important**: Because a random IV is used for every encryption, `encrypt/2`\nwill not produce the same ciphertext twice for the same value.","ref":"Cloak.Ciphers.AES.GCM.html#encrypt/2"},{"type":"module","title":"Cloak.Ciphers.Deprecated.AES.CTR","doc":"DEPRECATED version of the `Cloak.Ciphers.AES.CTR` cipher, for use in\nmigrating existing data to the new format used by `Cloak.Ciphers.AES.CTR`.","ref":"Cloak.Ciphers.Deprecated.AES.CTR.html"},{"type":"module","title":"Rationale - Cloak.Ciphers.Deprecated.AES.CTR","doc":"The old `Cloak.AES.CTR` cipher used the following format for ciphertext:\n\n    +---------------------------------------------------------+----------------------+\n    |                         HEADER                          |         BODY         |\n    +----------------------+------------------+---------------+----------------------+\n    | Module Tag (n bytes) | Key Tag (1 byte) | IV (16 bytes) | Ciphertext (n bytes) |\n    +----------------------+------------------+---------------+----------------------+\n\nThe new `Cloak.Ciphers.AES.CTR` implementation no longer prepends the \"Module Tag\"\ncomponent, and uses a new format as described in its docs. This cipher can\nassist in upgrading old ciphertext to the new format.\n\nSee the [Upgrading from 0.6.x](0.6.x_to_0.7.x.html) guide for usage.","ref":"Cloak.Ciphers.Deprecated.AES.CTR.html#module-rationale"},{"type":"module","title":"Cloak.Ciphers.Deprecated.AES.GCM","doc":"DEPRECATED version of the `Cloak.Ciphers.AES.GCM` cipher, for use in\nmigrating existing data to the new format used by `Cloak.Ciphers.AES.GCM`.","ref":"Cloak.Ciphers.Deprecated.AES.GCM.html"},{"type":"module","title":"Rationale - Cloak.Ciphers.Deprecated.AES.GCM","doc":"The old `Cloak.AES.GCM` cipher used the following format for ciphertext:\n\n    +---------------------------------------------------------+----------------------+\n    |                         HEADER                          |         BODY         |\n    +----------------------+------------------+---------------+----------------------+\n    | Module Tag (n bytes) | Key Tag (1 byte) | IV (16 bytes) | Ciphertext (n bytes) |\n    +----------------------+------------------+---------------+----------------------+\n\nThe new `Cloak.Ciphers.AES.GCM` implementation no longer prepends the \"Module Tag\"\ncomponent, and uses a new format as described in its docs. This cipher can\nassist in upgrading old ciphertext to the new format.\n\nSee the [Upgrading from 0.6.x](0.6.x_to_0.7.x.html) guide for usage.","ref":"Cloak.Ciphers.Deprecated.AES.GCM.html#module-rationale"},{"type":"exception","title":"Cloak.InvalidConfig","doc":"Raised when there is invalid configuration.","ref":"Cloak.InvalidConfig.html"},{"type":"exception","title":"Cloak.MissingCipher","doc":"Indicates that no cipher was found to encrypt/decrypt a given text.","ref":"Cloak.MissingCipher.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n## [v1.1.3](https://github.com/danielberkompas/cloak/tree/v1.1.3) (2024-04-06)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v1.1.2...v1.1.3)\n\n**Fixed bugs:**\n\n- Clock with changeset issue  [\\#115](https://github.com/danielberkompas/cloak/issues/115)\n- {:badarg, {'aead.c', 90}, 'Unknown cipher or invalid key size'} [\\#113](https://github.com/danielberkompas/cloak/issues/113)\n\n**Closed issues:**\n\n- How to use Cloak in ecto queries? [\\#122](https://github.com/danielberkompas/cloak/issues/122)\n- \\(UndefinedFunctionError\\) function :crypto.stream\\_init/3 is undefined or private [\\#118](https://github.com/danielberkompas/cloak/issues/118)\n\n**Merged pull requests:**\n\n- Fix build on SemaphoreCI [\\#124](https://github.com/danielberkompas/cloak/pull/124) ([danielberkompas](https://github.com/danielberkompas))\n- Allow vault gen server to return other responses [\\#123](https://github.com/danielberkompas/cloak/pull/123) ([spencerdcarlson](https://github.com/spencerdcarlson))\n- Update cheatsheet to fix mix config [\\#121](https://github.com/danielberkompas/cloak/pull/121) ([gworkman](https://github.com/gworkman))\n- Misc doc changes [\\#110](https://github.com/danielberkompas/cloak/pull/110) ([kianmeng](https://github.com/kianmeng))\n- Fix example code for Base.decode64! [\\#109](https://github.com/danielberkompas/cloak/pull/109) ([blunckr](https://github.com/blunckr))\n\n## [v1.1.2](https://github.com/danielberkompas/cloak/tree/v1.1.2) (2022-06-17)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v1.1.1...v1.1.2)\n\n**Closed issues:**\n\n- Accepting function as `key` configuration [\\#112](https://github.com/danielberkompas/cloak/issues/112)\n- ETS problem with Clock as application in umbrella project [\\#111](https://github.com/danielberkompas/cloak/issues/111)\n- Cloak relies in deprecated :crypto API that will be removed in OTP 24 [\\#102](https://github.com/danielberkompas/cloak/issues/102)\n\n**Merged pull requests:**\n\n- Run tests against Elixir 1.13 [\\#116](https://github.com/danielberkompas/cloak/pull/116) ([danielberkompas](https://github.com/danielberkompas))\n\n## [v1.1.1](https://github.com/danielberkompas/cloak/tree/v1.1.1) (2021-06-05)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v1.1.0...v1.1.1)\n\n**Merged pull requests:**\n\n- Remove unused pbkdf2 dependency [\\#107](https://github.com/danielberkompas/cloak/pull/107) ([danielberkompas](https://github.com/danielberkompas))\n\n## [v1.1.0](https://github.com/danielberkompas/cloak/tree/v1.1.0) (2021-06-05)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v1.0.3...v1.1.0)\n\n**Merged pull requests:**\n\n- Prepare for 1.1.0 release [\\#106](https://github.com/danielberkompas/cloak/pull/106) ([danielberkompas](https://github.com/danielberkompas))\n- Add regression tests to prepare for OTP 24 [\\#105](https://github.com/danielberkompas/cloak/pull/105) ([danielberkompas](https://github.com/danielberkompas))\n- Erlang 24 Support [\\#104](https://github.com/danielberkompas/cloak/pull/104) ([mjquinlan2000](https://github.com/mjquinlan2000))\n- Update Upgrade Documentation Link [\\#101](https://github.com/danielberkompas/cloak/pull/101) ([kevin-j-m](https://github.com/kevin-j-m))\n\n## [v1.0.3](https://github.com/danielberkompas/cloak/tree/v1.0.3) (2020-10-20)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v1.0.2...v1.0.3)\n\n## [v1.0.2](https://github.com/danielberkompas/cloak/tree/v1.0.2) (2020-02-01)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v1.0.1...v1.0.2)\n\n**Closed issues:**\n\n- AES GCM should use a 12 byte nonce instead of 16 [\\#93](https://github.com/danielberkompas/cloak/issues/93)\n\n**Merged pull requests:**\n\n- \\[\\#93\\] Add option to specify iv length for AES.GCM [\\#95](https://github.com/danielberkompas/cloak/pull/95) ([danielberkompas](https://github.com/danielberkompas))\n\n## [v1.0.1](https://github.com/danielberkompas/cloak/tree/v1.0.1) (2020-01-29)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v1.0.0...v1.0.1)\n\n**Closed issues:**\n\n- MyApp.Vault.encrypt\\(\"plaintext\"\\) failing [\\#92](https://github.com/danielberkompas/cloak/issues/92)\n\n**Merged pull requests:**\n\n- Clean up and update dev deps [\\#94](https://github.com/danielberkompas/cloak/pull/94) ([chulkilee](https://github.com/chulkilee))\n\n## [v1.0.0](https://github.com/danielberkompas/cloak/tree/v1.0.0) (2019-07-31)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v1.0.0-alpha.0...v1.0.0)\n\n**Closed issues:**\n\n- Outstanding work for 1.0? [\\#91](https://github.com/danielberkompas/cloak/issues/91)\n\n**Merged pull requests:**\n\n- Correct code example on upgrading guide [\\#90](https://github.com/danielberkompas/cloak/pull/90) ([ruan-brandao](https://github.com/ruan-brandao))\n\n## [v1.0.0-alpha.0](https://github.com/danielberkompas/cloak/tree/v1.0.0-alpha.0) (2018-12-31)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.9.2...v1.0.0-alpha.0)\n\n**Breaking changes:**\n\n- Extract Ecto components to cloak\\_ecto hex package [\\#77](https://github.com/danielberkompas/cloak/issues/77)\n\n**Merged pull requests:**\n\n- Extract Ecto components to cloak\\_ecto [\\#89](https://github.com/danielberkompas/cloak/pull/89) ([danielberkompas](https://github.com/danielberkompas))\n\n## [v0.9.2](https://github.com/danielberkompas/cloak/tree/v0.9.2) (2018-12-26)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.9.1...v0.9.2)\n\n**Merged pull requests:**\n\n- Fix typo & add better error msg for PBKDF2 algorithm [\\#88](https://github.com/danielberkompas/cloak/pull/88) ([jc00ke](https://github.com/jc00ke))\n- Improve PBKDF2 config error messages [\\#87](https://github.com/danielberkompas/cloak/pull/87) ([jc00ke](https://github.com/jc00ke))\n\n## [v0.9.1](https://github.com/danielberkompas/cloak/tree/v0.9.1) (2018-10-07)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.9.0...v0.9.1)\n\n**Closed issues:**\n\n- Testing with hashed fields fails to equal fixture [\\#85](https://github.com/danielberkompas/cloak/issues/85)\n- Migrator Embedded Schema Error [\\#81](https://github.com/danielberkompas/cloak/issues/81)\n\n**Merged pull requests:**\n\n- \\[\\#81\\] Fix migrator with embedded schemas [\\#83](https://github.com/danielberkompas/cloak/pull/83) ([danielberkompas](https://github.com/danielberkompas))\n- Migration task succeeds on empty table [\\#82](https://github.com/danielberkompas/cloak/pull/82) ([kevin-j-m](https://github.com/kevin-j-m))\n- Update docs [\\#80](https://github.com/danielberkompas/cloak/pull/80) ([hl](https://github.com/hl))\n- Upgrade Flow to 0.14 [\\#79](https://github.com/danielberkompas/cloak/pull/79) ([mikeastock](https://github.com/mikeastock))\n\n## [v0.9.0](https://github.com/danielberkompas/cloak/tree/v0.9.0) (2018-09-29)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.8.0...v0.9.0)\n\n**Breaking changes:**\n\n- Make Cloak.Vault a GenServer [\\#75](https://github.com/danielberkompas/cloak/issues/75)\n\n**Merged pull requests:**\n\n- \\[\\#75\\] Make Cloak.Vault a GenServer [\\#78](https://github.com/danielberkompas/cloak/pull/78) ([danielberkompas](https://github.com/danielberkompas))\n\n## [v0.8.0](https://github.com/danielberkompas/cloak/tree/v0.8.0) (2018-09-22)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.7.0...v0.8.0)\n\n**Closed issues:**\n\n- convert Cloak errors into Ecto errors [\\#74](https://github.com/danielberkompas/cloak/issues/74)\n- Support migrations with non-integer `:id` field [\\#70](https://github.com/danielberkompas/cloak/issues/70)\n- Roadmap [\\#66](https://github.com/danielberkompas/cloak/issues/66)\n\n**Merged pull requests:**\n\n- Support :binary\\_ids via cursor paging [\\#73](https://github.com/danielberkompas/cloak/pull/73) ([danielberkompas](https://github.com/danielberkompas))\n- Add a link to the 0.6.x to 0.7.x migration guide to readme [\\#72](https://github.com/danielberkompas/cloak/pull/72) ([4141done](https://github.com/4141done))\n- Do not fail migration task when running on schema linked to empty table [\\#69](https://github.com/danielberkompas/cloak/pull/69) ([tizpuppi](https://github.com/tizpuppi))\n\n## [v0.7.0](https://github.com/danielberkompas/cloak/tree/v0.7.0) (2018-08-23)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.7.0-alpha.2...v0.7.0)\n\n**Merged pull requests:**\n\n- Prepare for 0.7 release [\\#68](https://github.com/danielberkompas/cloak/pull/68) ([danielberkompas](https://github.com/danielberkompas))\n- Add support for PBKDF2 for blind indexing [\\#67](https://github.com/danielberkompas/cloak/pull/67) ([michaelherold](https://github.com/michaelherold))\n\n## [v0.7.0-alpha.2](https://github.com/danielberkompas/cloak/tree/v0.7.0-alpha.2) (2018-04-25)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.7.0-alpha.1...v0.7.0-alpha.2)\n\n**Closed issues:**\n\n- \\(Postgrex.Error\\) ERROR 22021 \\(character\\_not\\_in\\_repertoire\\) [\\#65](https://github.com/danielberkompas/cloak/issues/65)\n\n**Merged pull requests:**\n\n- Update install.md [\\#64](https://github.com/danielberkompas/cloak/pull/64) ([mhussa](https://github.com/mhussa))\n- Improve nil support in Ecto types [\\#63](https://github.com/danielberkompas/cloak/pull/63) ([danielberkompas](https://github.com/danielberkompas))\n\n## [v0.7.0-alpha.1](https://github.com/danielberkompas/cloak/tree/v0.7.0-alpha.1) (2018-03-21)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.6.2...v0.7.0-alpha.1)\n\n**Closed issues:**\n\n- Cloak.AES.CTR `encrypt/2` not explicitly called? [\\#38](https://github.com/danielberkompas/cloak/issues/38)\n- Recommendations for data migrations [\\#29](https://github.com/danielberkompas/cloak/issues/29)\n\n**Merged pull requests:**\n\n- Rewrite mix.cloak.migrate [\\#62](https://github.com/danielberkompas/cloak/pull/62) ([danielberkompas](https://github.com/danielberkompas))\n- Vaults [\\#60](https://github.com/danielberkompas/cloak/pull/60) ([danielberkompas](https://github.com/danielberkompas))\n\n## [v0.6.2](https://github.com/danielberkompas/cloak/tree/v0.6.2) (2018-03-19)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.6.1...v0.6.2)\n\n**Closed issues:**\n\n- Is poison optional? [\\#59](https://github.com/danielberkompas/cloak/issues/59)\n- Clarification on usage of 'default' in documents [\\#58](https://github.com/danielberkompas/cloak/issues/58)\n- Argument error [\\#57](https://github.com/danielberkompas/cloak/issues/57)\n- Use with mnesia? [\\#55](https://github.com/danielberkompas/cloak/issues/55)\n\n**Merged pull requests:**\n\n- \\[Documentation\\] Fix small typo: SHA265 -\\> SHA256 [\\#56](https://github.com/danielberkompas/cloak/pull/56) ([connorlay](https://github.com/connorlay))\n\n## [v0.6.1](https://github.com/danielberkompas/cloak/tree/v0.6.1) (2018-02-25)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.6.0...v0.6.1)\n\n**Merged pull requests:**\n\n- bugfix\\(cipher?\\): Rescue from UndefinedFunctionError instead of using … [\\#54](https://github.com/danielberkompas/cloak/pull/54) ([tomciopp](https://github.com/tomciopp))\n\n## [v0.6.0](https://github.com/danielberkompas/cloak/tree/v0.6.0) (2018-02-24)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.5.0...v0.6.0)\n\n**Merged pull requests:**\n\n- Add support for encrypted Ecto String and Integer Array types [\\#53](https://github.com/danielberkompas/cloak/pull/53) ([boydm](https://github.com/boydm))\n- perf\\(Cloak.Config.all/0\\): remove calls to ensure\\_loaded when checking… [\\#51](https://github.com/danielberkompas/cloak/pull/51) ([tomciopp](https://github.com/tomciopp))\n\n## [v0.5.0](https://github.com/danielberkompas/cloak/tree/v0.5.0) (2018-01-06)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.4.0...v0.5.0)\n\n**Closed issues:**\n\n- Migrations in multi tenant applications [\\#43](https://github.com/danielberkompas/cloak/issues/43)\n- Proposal AES GCM cipher as default Cipher [\\#33](https://github.com/danielberkompas/cloak/issues/33)\n\n**Merged pull requests:**\n\n- Add EncryptedTimeField [\\#50](https://github.com/danielberkompas/cloak/pull/50) ([danielberkompas](https://github.com/danielberkompas))\n- Update the documentation sidebar [\\#49](https://github.com/danielberkompas/cloak/pull/49) ([danielberkompas](https://github.com/danielberkompas))\n- Add DateTime field, better docs [\\#48](https://github.com/danielberkompas/cloak/pull/48) ([danielberkompas](https://github.com/danielberkompas))\n- refactor\\(Cloak.Config.all/0\\): Find ciphers by checking if the module … [\\#47](https://github.com/danielberkompas/cloak/pull/47) ([tomciopp](https://github.com/tomciopp))\n- feat\\(Date Types\\): add EncryptedDateField, and EncryptedNaiveDatetimeF… [\\#46](https://github.com/danielberkompas/cloak/pull/46) ([tomciopp](https://github.com/tomciopp))\n\n## [v0.4.0](https://github.com/danielberkompas/cloak/tree/v0.4.0) (2018-01-02)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.3.3...v0.4.0)\n\n**Fixed bugs:**\n\n- instructions to cloak plain text fields [\\#17](https://github.com/danielberkompas/cloak/issues/17)\n\n**Closed issues:**\n\n- Make json library configurable [\\#44](https://github.com/danielberkompas/cloak/issues/44)\n- Encryption Arguement Error [\\#41](https://github.com/danielberkompas/cloak/issues/41)\n- \\*\\* \\(MatchError\\) no match of right hand side value: false [\\#40](https://github.com/danielberkompas/cloak/issues/40)\n- Proposal / Forthcoming PRs: Multiple Ciphers With Keys [\\#36](https://github.com/danielberkompas/cloak/issues/36)\n\n**Merged pull requests:**\n\n- Modernize codebase for new release [\\#45](https://github.com/danielberkompas/cloak/pull/45) ([danielberkompas](https://github.com/danielberkompas))\n- \\[\\#33\\] Add galois counter mode cipher [\\#42](https://github.com/danielberkompas/cloak/pull/42) ([tomciopp](https://github.com/tomciopp))\n- Fix typespecs / dialyzer errors across the board [\\#39](https://github.com/danielberkompas/cloak/pull/39) ([asummers](https://github.com/asummers))\n- Allow multiple concurrent ciphers to encrypt [\\#37](https://github.com/danielberkompas/cloak/pull/37) ([asummers](https://github.com/asummers))\n\n## [v0.3.3](https://github.com/danielberkompas/cloak/tree/v0.3.3) (2017-09-08)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.3.2...v0.3.3)\n\n**Closed issues:**\n\n- EncryptedIntegerField and EncryptedFloat field do not cast properly [\\#35](https://github.com/danielberkompas/cloak/issues/35)\n\n## [v0.3.2](https://github.com/danielberkompas/cloak/tree/v0.3.2) (2017-08-05)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.3.0...v0.3.2)\n\n**Closed issues:**\n\n- Compile warnings [\\#34](https://github.com/danielberkompas/cloak/issues/34)\n- Allow key to be in Application env [\\#31](https://github.com/danielberkompas/cloak/issues/31)\n- Add salt option to cloak for increased security [\\#26](https://github.com/danielberkompas/cloak/issues/26)\n- How to encrypt with user's own key instead of key from config? [\\#24](https://github.com/danielberkompas/cloak/issues/24)\n- index on encryption\\_version [\\#23](https://github.com/danielberkompas/cloak/issues/23)\n- Adding interoperability with attr\\_encrypted? [\\#22](https://github.com/danielberkompas/cloak/issues/22)\n- Provide Vault Transit cipher [\\#21](https://github.com/danielberkompas/cloak/issues/21)\n\n**Merged pull requests:**\n\n- Add ability to pull cipher key from OTP app env [\\#32](https://github.com/danielberkompas/cloak/pull/32) ([tielur](https://github.com/tielur))\n- Add `prepare_changes` example to module doc for use with ecto 2.0 [\\#27](https://github.com/danielberkompas/cloak/pull/27) ([kgautreaux](https://github.com/kgautreaux))\n- clean up compile warnings under Elixir 1.4 [\\#25](https://github.com/danielberkompas/cloak/pull/25) ([boydm](https://github.com/boydm))\n\n## [v0.3.0](https://github.com/danielberkompas/cloak/tree/v0.3.0) (2016-09-16)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.2.3...v0.3.0)\n\n**Merged pull requests:**\n\n- Rely on configuration at runtime [\\#20](https://github.com/danielberkompas/cloak/pull/20) ([danielberkompas](https://github.com/danielberkompas))\n\n## [v0.2.3](https://github.com/danielberkompas/cloak/tree/v0.2.3) (2016-09-16)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.2.2...v0.2.3)\n\n**Fixed bugs:**\n\n- mix cloak.migrate doesn't update encryption\\_version field [\\#16](https://github.com/danielberkompas/cloak/issues/16)\n\n**Closed issues:**\n\n- cloak migration not running [\\#18](https://github.com/danielberkompas/cloak/issues/18)\n- Issue integration with Ecto2? [\\#12](https://github.com/danielberkompas/cloak/issues/12)\n- Support for Ecto 2.0 [\\#10](https://github.com/danielberkompas/cloak/issues/10)\n- :system AES keys aren't getting base64 decoded [\\#9](https://github.com/danielberkompas/cloak/issues/9)\n\n**Merged pull requests:**\n\n- \\[\\#16\\] Update encryption version in cloak.migrate [\\#19](https://github.com/danielberkompas/cloak/pull/19) ([danielberkompas](https://github.com/danielberkompas))\n- Fix validation bug in cloak.migrate [\\#15](https://github.com/danielberkompas/cloak/pull/15) ([bgeihsgt](https://github.com/bgeihsgt))\n- Add key generation example to README. [\\#14](https://github.com/danielberkompas/cloak/pull/14) ([ghost](https://github.com/ghost))\n- Add deps badge to README [\\#13](https://github.com/danielberkompas/cloak/pull/13) ([rrrene](https://github.com/rrrene))\n\n## [v0.2.2](https://github.com/danielberkompas/cloak/tree/v0.2.2) (2016-05-20)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.2.1...v0.2.2)\n\n**Closed issues:**\n\n- Cloak.EncryptedMapField strange Poison errors [\\#6](https://github.com/danielberkompas/cloak/issues/6)\n- How to generate keys? [\\#5](https://github.com/danielberkompas/cloak/issues/5)\n\n**Merged pull requests:**\n\n- Decode AES environment variable keys from base64 [\\#11](https://github.com/danielberkompas/cloak/pull/11) ([philss](https://github.com/philss))\n\n## [v0.2.1](https://github.com/danielberkompas/cloak/tree/v0.2.1) (2016-04-06)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.2.0...v0.2.1)\n\n**Merged pull requests:**\n\n- Update examples to reflect the remotion of Cloak.Model [\\#4](https://github.com/danielberkompas/cloak/pull/4) ([philss](https://github.com/philss))\n\n## [v0.2.0](https://github.com/danielberkompas/cloak/tree/v0.2.0) (2015-12-16)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.1.0...v0.2.0)\n\n**Merged pull requests:**\n\n- Added sample migration to README [\\#3](https://github.com/danielberkompas/cloak/pull/3) ([mspanc](https://github.com/mspanc))\n- Added ability to store keys in the environment variables [\\#2](https://github.com/danielberkompas/cloak/pull/2) ([mspanc](https://github.com/mspanc))\n- Inform that key must be 16, 24 or 32 bytes long [\\#1](https://github.com/danielberkompas/cloak/pull/1) ([mspanc](https://github.com/mspanc))\n\n## [v0.1.0](https://github.com/danielberkompas/cloak/tree/v0.1.0) (2015-09-22)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/v0.1.0-pre...v0.1.0)\n\n## [v0.1.0-pre](https://github.com/danielberkompas/cloak/tree/v0.1.0-pre) (2015-09-16)\n\n[Full Changelog](https://github.com/danielberkompas/cloak/compare/2bd17019e285b55c5c218cc842537bf9280f24c3...v0.1.0-pre)\n\n\n\n\\* *This Changelog was automatically generated by [github_changelog_generator](https://github.com/github-changelog-generator/github-changelog-generator)*","ref":"changelog.html"},{"type":"extras","title":"License","doc":"# The MIT License (MIT)\n\nCopyright (c) 2015 Daniel Berkompas\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"Cloak\n======\n\n[![Hex Version](http://img.shields.io/hexpm/v/cloak.svg)](https://hex.pm/packages/cloak)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/cloak/)\n[![Total Download](https://img.shields.io/hexpm/dt/cloak.svg)](https://hex.pm/packages/cloak)\n[![License](https://img.shields.io/hexpm/l/cloak.svg)](https://github.com/danielberkompas/cloak/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/danielberkompas/cloak.svg)](https://github.com/danielberkompas/cloak/commits/master)\n[![Build Status](https://danielberkompas.semaphoreci.com/badges/cloak/branches/master.svg?style=shields)](https://danielberkompas.semaphoreci.com/projects/cloak)\n[![Inline docs](http://inch-ci.org/github/danielberkompas/cloak.svg?branch=master)](http://inch-ci.org/github/danielberkompas/cloak)\n[![Coverage Status](https://coveralls.io/repos/github/danielberkompas/cloak/badge.svg?branch=migrate)](https://coveralls.io/github/danielberkompas/cloak?branch=migrate)\n\n\nCloak is an Elixir encryption library that implements several best practices\nand conveniences for Elixir developers:\n\n- Random IVs\n- Tagged ciphertexts\n- Elixir-native configuration","ref":"readme.html"},{"type":"extras","title":"Documentation - Overview","doc":"- [Hex Documentation](https://hexdocs.pm/cloak) (Includes installation guide)\n- [How to upgrade from Cloak 0.9.x to 1.0.x](https://hexdocs.pm/cloak/0-9-x_to_1-0-x.html)","ref":"readme.html#documentation"},{"type":"extras","title":"Examples - Overview","doc":"#","ref":"readme.html#examples"},{"type":"extras","title":"Encrypt / Decrypt - Overview","doc":"```elixir\n{:ok, ciphertext} = MyApp.Vault.encrypt(\"plaintext\")\n# => {:ok, <<1, 10, 65, 69, 83, 46, 71, 67, 77, 46, 86, 49, 45, 1, 250, 221,\n# =>  189, 64, 26, 214, 26, 147, 171, 101, 181, 158, 224, 117, 10, 254, 140, 207,\n# =>  215, 98, 208, 208, 174, 162, 33, 197, 179, 56, 236, 71, 81, 67, 85, 229,\n# =>  ...>>}\n\nMyApp.Vault.decrypt(ciphertext)\n# => {:ok, \"plaintext\"}\n```\n\n#","ref":"readme.html#encrypt-decrypt"},{"type":"extras","title":"Reencrypt With New Algorithm/Key - Overview","doc":"```elixir\n\"plaintext\"\n|> MyApp.Vault.encrypt!(:aes_256)\n|> MyApp.Vault.decrypt!()\n|> MyApp.Vault.encrypt!(:aes_256)\n|> MyApp.Vault.decrypt!()\n# => \"plaintext\"\n```\n\n#","ref":"readme.html#reencrypt-with-new-algorithm-key"},{"type":"extras","title":"Configuration - Overview","doc":"```elixir\nconfig :my_app, MyApp.Vault,\n  ciphers: [\n    # In AES.GCM, it is important to specify 12-byte IV length for\n    # interoperability with other encryption software. See this GitHub issue\n    # for more details: https://github.com/danielberkompas/cloak/issues/93\n    #\n    # In Cloak 2.0, this will be the default iv length for AES.GCM.\n    aes_gcm: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V1\", key: <<...>>, iv_length: 12},\n    aes_ctr: {Cloak.Ciphers.AES.CTR, tag: \"AES.CTR.V1\", key: <<...>>}\n  ]\n```","ref":"readme.html#configuration"},{"type":"extras","title":"Features - Overview","doc":"#","ref":"readme.html#features"},{"type":"extras","title":"Random Initialization Vectors (IV) - Overview","doc":"Every strong encryption algorithm recommends unique initialization vectors.\nCloak automatically generates unique vectors using\n`:crypto.strong_rand_bytes`, and includes the IV in the ciphertext.\nThis greatly simplifies storage and is not a security risk.\n\n#","ref":"readme.html#random-initialization-vectors-iv"},{"type":"extras","title":"Tagged Ciphertext - Overview","doc":"Each ciphertext contains metadata about the algorithm and key which was used\nto encrypt it. This allows Cloak to automatically select the correct key and\nalgorithm to use for decryption for any given ciphertext.\n\nThis makes key rotation much easier, because you can easily tell whether any\ngiven ciphertext is using the old key or the new key.\n\n#","ref":"readme.html#tagged-ciphertext"},{"type":"extras","title":"Elixir-Native Configuration - Overview","doc":"Cloak works through `Vault` modules which you define in your app, and add\nto your supervision tree.\n\nYou can have as many vaults as you wish running simultaneously in your\nproject. (This works well with umbrella apps, or any runtime environment\nwhere you have multiple OTP apps using Cloak)\n\n#","ref":"readme.html#elixir-native-configuration"},{"type":"extras","title":"Ecto Support - Overview","doc":"You can use Cloak to transparently encrypt Ecto fields, using\n[`cloak_ecto`](https://hex.pm/packages/cloak_ecto).","ref":"readme.html#ecto-support"},{"type":"extras","title":"Security Notes - Overview","doc":"- Cloak is built on Erlang's `crypto` library, and therefore inherits its security.\n- You can implement your own cipher modules to use with Cloak, which may use any other encryption algorithms of your choice.","ref":"readme.html#security-notes"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2015 Daniel Berkompas\n\nThis work is free. You can redistribute it and/or modify it under the\nterms of the MIT License. See the [LICENSE.md](./LICENSE.md) file for more details.","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Cheatsheet","doc":"","ref":"cheatsheet.html"},{"type":"extras","title":"Setup - Cheatsheet","doc":"{: .col-2}\n\n#","ref":"cheatsheet.html#setup"},{"type":"extras","title":"Define a vault module - Cheatsheet","doc":"This is the module you will use to encrypt/decrypt your ciphertext. You can have more than one!\n\n```elixir\ndefmodule MyApp.MyVault do\n  use Cloak.Vault, otp_app: :my_app\nend\n```\n\n#","ref":"cheatsheet.html#define-a-vault-module"},{"type":"extras","title":"Generate an encryption key - Cheatsheet","doc":"This will generate a 256-bit key in Base64 encoding that you can store in an environment variable.\n\n```elixir\n32\n|> :crypto.strong_rand_bytes()\n|> Base.encode64\n```\n\n#","ref":"cheatsheet.html#generate-an-encryption-key"},{"type":"extras","title":"Custom Cipher Module - Cheatsheet","doc":"If you don't want to use any of the included cipher modules, you can define your own. See `Cloak.Cipher`.\n\n```elixir\ndefmodule MyApp.MyCipher do\n  @behaviour Cloak.Cipher\n\n  @impl true\n  def encrypt(plaintext, opts) do\n    # your logic here\n  end\n\n  @impl true\n  def decrypt(ciphertext, opts) do\n    # your logic here\n  end\n\n  @impl true\n  def can_decrypt?(ciphertext, opts) do\n    # return a boolean\n  end\nend\n```","ref":"cheatsheet.html#custom-cipher-module"},{"type":"extras","title":"Configure - Cheatsheet","doc":"{: .col-2}\n\n#","ref":"cheatsheet.html#configure"},{"type":"extras","title":"Using `Config` - Cheatsheet","doc":"This is the easiest option, and the vault will automatically read this configuration.\n\n```elixir\n# config/runtime.exs\nconfig :my_app, MyApp.MyVault,\n    ciphers: [\n      default: {Cloak.Ciphers.AES.GCM, \n      tag: \"AES.GCM.V1\", \n      key: Base.decode64!(\"your-key-here\"),\n      iv_length: 12}\n    ]\n```\n\n#","ref":"cheatsheet.html#using-config"},{"type":"extras","title":"Inside the Vault module - Cheatsheet","doc":"This is best if you need to fetch the key from some custom store before you use it. Add these lines to your vault module.\n\n```elixir\ndefmodule MyApp.Vault do\n  use Cloak.Vault, otp_app: :my_app\n\n  @impl GenServer\n  def init(config) do\n    config =\n      Keyword.put(config, :ciphers, [\n        default: {\n          Cloak.Ciphers.AES.GCM, \n          tag: \"AES.GCM.V1\", \n          key: decode_env!(\"CLOAK_KEY\"),\n          iv_length: 12\n        }\n      ])\n\n    {:ok, config}\n  end\n\n  defp decode_env!(var) do\n    var\n    |> System.get_env()\n    |> Base.decode64!()\n  end\nend\n```","ref":"cheatsheet.html#inside-the-vault-module"},{"type":"extras","title":"Usage - Cheatsheet","doc":"{: .col-2}\n\n#","ref":"cheatsheet.html#usage"},{"type":"extras","title":"Encrypt - Cheatsheet","doc":"##","ref":"cheatsheet.html#encrypt"},{"type":"extras","title":"Safe: return any errors - Cheatsheet","doc":"```elixir\nMyApp.MyVault.encrypt(\"plaintext\")\n# => {:ok, \"ciphertext\"}\n```\n\n##","ref":"cheatsheet.html#safe-return-any-errors"},{"type":"extras","title":"Unsafe: raise any errors - Cheatsheet","doc":"```elixir\nMyApp.MyVault.encrypt!(\"plaintext\")\n# => \"ciphertext\"\n```\n\n\n#","ref":"cheatsheet.html#unsafe-raise-any-errors"},{"type":"extras","title":"Decrypt - Cheatsheet","doc":"##","ref":"cheatsheet.html#decrypt"},{"type":"extras","title":"Safe: return any errors - Cheatsheet","doc":"```elixir\nMyApp.MyVault.decrypt(\"ciphertext\")\n# => {:ok, \"plaintext\"}\n```\n\n##","ref":"cheatsheet.html#safe-return-any-errors"},{"type":"extras","title":"Unsafe: raise any errors - Cheatsheet","doc":"```elixir\nMyApp.MyVault.decrypt!(\"ciphertext\")\n# => \"plaintext\"\n```","ref":"cheatsheet.html#unsafe-raise-any-errors"},{"type":"extras","title":"Usage with Ecto - Cheatsheet","doc":"See the [`cloak_ecto`](https://hexdocs.pm/cloak_ecto) library! Here's a preview:\n\n```elixir\ndefmodule MyApp.MySchema do\n  use Ecto.Schema\n\n  schema \"table\" do\n    field :encrypted_field, MyApp.Encrypted.Binary\n  end\nend\n```","ref":"cheatsheet.html#usage-with-ecto"},{"type":"extras","title":"Install Cloak","doc":"# Installing Cloak\n\nThis guide will walk you through installing Cloak in your project.\n\n#","ref":"install.html"},{"type":"extras","title":"Add the Dependency - Install Cloak","doc":"First, add `:cloak` to your dependencies in `mix.exs`:\n\n    {:cloak, \"1.1.1\"}\n\nRun `mix deps.get` to fetch the dependency.\n\n#","ref":"install.html#add-the-dependency"},{"type":"extras","title":"Generate a Key - Install Cloak","doc":"You'll need a secret key for encryption. This is easy to generate in the\nIEx console.\n\n    $ iex\n    iex> 32 |> :crypto.strong_rand_bytes() |> Base.encode64()\n    \"aJ7HcM24BcyiwsAvRsa3EG3jcvaFWooyQJ+91OO7bRU=\"\n\nThis will generate a relatively strong encryption 256-bit encryption\nkey encoded with Base64.\n\n#","ref":"install.html#generate-a-key"},{"type":"extras","title":"Create a Vault - Install Cloak","doc":"Next, create a `Cloak.Vault` for your project.\n\n    defmodule MyApp.Vault do\n      use Cloak.Vault, otp_app: :my_app\n    end\n\nConfigure it as shown in the `Cloak.Vault` documentation, with at least one\nactive cipher. Note that the `:key` needs to be decoded from Base64 encoding into\nits raw binary form.\n\n    config :my_app, MyApp.Vault,\n      ciphers: [\n        default: {\n          Cloak.Ciphers.AES.GCM, \n          tag: \"AES.GCM.V1\", \n          key: Base.decode64!(\"your-key-here\"),\n          # In AES.GCM, it is important to specify 12-byte IV length for\n          # interoperability with other encryption software. See this GitHub\n          # issue for more details:\n          # https://github.com/danielberkompas/cloak/issues/93\n          # \n          # In Cloak 2.0, this will be the default iv length for AES.GCM.\n          iv_length: 12\n        }\n      ]\n\nIf you want to fetch keys from system vars, you should use the `init/1` callback\nto configure the vault instead:\n\n    # Assumes that you have a CLOAK_KEY environment variable containing a key in\n    # Base64 encoding.\n    #\n    # export CLOAK_KEY=\"A7x+qcFD9yeRfl3GohiOFZM5bNCdHNu27B0Ozv8X4dE=\"\n\n    defmodule MyApp.Vault do\n      use Cloak.Vault, otp_app: :my_app\n\n      @impl GenServer\n      def init(config) do\n        config =\n          Keyword.put(config, :ciphers, [\n            default: {\n              Cloak.Ciphers.AES.GCM, \n              tag: \"AES.GCM.V1\", \n              key: decode_env!(\"CLOAK_KEY\"),\n              iv_length: 12\n            }\n          ])\n\n        {:ok, config}\n      end\n\n      defp decode_env!(var) do\n        var\n        |> System.get_env()\n        |> Base.decode64!()\n      end\n    end\n\nFinally, add your vault to your supervision tree.\n\n    children = [\n      MyApp.Vault\n    ]","ref":"install.html#create-a-vault"},{"type":"extras","title":"Usage - Install Cloak","doc":"You can now encrypt and decrypt values using your Vault.\n\n    {:ok, ciphertext} = MyApp.Vault.encrypt(\"plaintext\") \n    # => {:ok, <<1, 10, 65, 69, 83, 46, 71, 67, 77, 46, 86, 49, 93, 140, 255, 234,\n    1, 195, 125, 112, 121, 186, 169, 185, 129, 122, 237, 161, 160, 24, 166,\n    48, 224, 230, 53, 194, 251, 175, 215, 10, 186, 130, 61, 230, 176, 102,\n    213, 209, ...>>}\n\n    MyApp.Vault.decrypt(ciphertext)\n    {:ok, \"plaintext\"}\n\nBy default, the first configured key will be used. You can use a specific key\nto use by referencing its label:\n\n    MyApp.Vault.encrypt(\"plaintext\", :default)\n\nDecryption will use the metadata embedded in the ciphertext to decide which\nconfigured key to use.","ref":"install.html#usage"},{"type":"extras","title":"Usage with Ecto - Install Cloak","doc":"If you want to use Cloak to automatically encrypt and decrypt fields in your\n`Ecto` schemas, see [`cloak_ecto`](https://hex.pm/packages/cloak_ecto).","ref":"install.html#usage-with-ecto"},{"type":"extras","title":"Generate Encryption Keys","doc":"# How to Generate Encryption Keys\n\nThe easiest way to generate a key is via IEx.\n\n    $ iex\n    iex> 32 |> :crypto.strong_rand_bytes() |> Base.encode64()\n    \"HXCdm5z61eNgUpnXObJRv94k3JnKSrnfwppyb60nz6w=\"\n\nThis will generate a strong 256-bit key encoded with Base64.","ref":"generate_keys.html"},{"type":"extras","title":"0.9.x to 1.0.x","doc":"# How To Upgrade From 0.9.x to 1.0.x\n\nCloak 1.0.0 separated the `Ecto` components into a separate package:\n`cloak_ecto`. This required some breaking name changes.\n\n- Cloak's default JSON library is now `Jason`, not `Poison`\n- All `Ecto`-related modules now have the prefix `Cloak.Ecto`.\n- Cloak's `Ecto` types have a new naming scheme: `Cloak.Fields.Binary` became\n  `Cloak.Ecto.Binary`.\n- `mix cloak.migrate` has been renamed to `mix cloak.migrate.ecto`.\n\n#","ref":"0-9-x_to_1-0-x.html"},{"type":"extras","title":"Install `cloak_ecto` 1.0.0 - 0.9.x to 1.0.x","doc":"Replace your `cloak` dependency with `cloak_ecto`.\n\n    {:cloak_ecto, \"1.0.0\"}\n\n#","ref":"0-9-x_to_1-0-x.html#install-cloak_ecto-1-0-0"},{"type":"extras","title":"Configure JSON Library - 0.9.x to 1.0.x","doc":"If you have not specified the `:json_library` setting in your Vault, and want\nto continue using Poison, you will need to add that setting:\n\n    config :my_app, MyApp.Vault,\n      json_library: Poison,\n      # ...\n\nOtherwise, you will need to add `Jason` to your dependencies:\n\n    {:jason, \"~> 1.0\"}\n\n#","ref":"0-9-x_to_1-0-x.html#configure-json-library"},{"type":"extras","title":"Update Cloak Field References - 0.9.x to 1.0.x","doc":"Update all your Cloak field references from the old naming scheme to the new.\nFor example:\n\n    use Cloak.Fields.Binary, vault: MyApp.Vault\n\nShould now become:\n\n    use Cloak.Ecto.Binary, vault: MyApp.Vault\n\n#","ref":"0-9-x_to_1-0-x.html#update-cloak-field-references"},{"type":"extras","title":"Update Migration References - 0.9.x to 1.0.x","doc":"If you use `Cloak.Migrator` directly, rename your references to\n`Cloak.Ecto.Migrator` instead. If you use `mix cloak.migrate`, rename any\nreferences to `mix cloak.migrate.ecto` instead..","ref":"0-9-x_to_1-0-x.html#update-migration-references"},{"type":"extras","title":"0.8.x to 0.9.x","doc":"# How To Upgrade From 0.8.x to 0.9.x\n\nCloak `0.9.0` encrypts and decrypts data up to 200% faster than `0.8.0`.\nThis result is achieved by caching configuration data in an ETS table.\n\n- **Breaking Change**: You must now add your `Cloak.Vault` to your\n  supervision tree.","ref":"0-8-x_to_0-9-x.html"},{"type":"extras","title":"Update Your Dependency - 0.8.x to 0.9.x","doc":"Update your `cloak` dependency to `0.9.0` or higher:\n\n```elixir\n{:cloak, \"~> 0.9.2\"}\n```\n\n#","ref":"0-8-x_to_0-9-x.html#update-your-dependency"},{"type":"extras","title":"Supervise Your Vault - 0.8.x to 0.9.x","doc":"Add your vault to your supervision tree:\n\n    children = [\n      MyApp.Vault\n    ]\n\n#","ref":"0-8-x_to_0-9-x.html#supervise-your-vault"},{"type":"extras","title":"Update `init/1` callback implementation - 0.8.x to 0.9.x","doc":"If you are fetching the keys from system vars, you should change the implementation of the `init/1` callback to `GenServer.init/1`\n\n```elixir\ndefmodule MyApp.Vault do\n  use Cloak.Vault, otp_app: :my_app\n\n  @impl GenServer\n  def init(config) do\n    (...)\n  end\nend\n```","ref":"0-8-x_to_0-9-x.html#update-init-1-callback-implementation"},{"type":"extras","title":"0.7.x to 0.8.x","doc":"# How To Upgrade From 0.7.x to 0.8.x\n\nCloak `0.8` introduces a new feature with **no breaking changes**.\n\n- `mix cloak.migrate` now supports binary primary keys. It iterates\n  through database tables using a cursor, instead of min/max ID.\n\n- `Cloak.CustomCursor` allows you to customize fields used for the cursor.","ref":"0-7-x_to_0-8-x.html"},{"type":"extras","title":"Update Your Dependency - 0.7.x to 0.8.x","doc":"Update your `cloak` dependency to `0.8` or later.\n\n```elixir\n{:cloak, \"~> 0.8.0\"}\n```\n\nMost users will not need to do anything more. See the `Cloak.CustomCursor`\ndocs for more details.","ref":"0-7-x_to_0-8-x.html#update-your-dependency"},{"type":"extras","title":"0.6.x to 0.7.x","doc":"# How To Upgrade From 0.6.x to 0.7.x\n\nCloak 0.7 introduced a number of important changes.\n\n- Encryption is now performed through `Cloak.Vault` modules\n- Ciphertext no longer contains a `module_tag`\n- Ecto types are now local to your project\n- You no longer need an `:encryption_version` field","ref":"0-6-x_to_0-7-x.html"},{"type":"extras","title":"Install Cloak 0.7 - 0.6.x to 0.7.x","doc":"Update your `cloak` dependency to `0.7` or later.\n\n    {:cloak, \"~> 0.7.0\"}","ref":"0-6-x_to_0-7-x.html#install-cloak-0-7"},{"type":"extras","title":"Create a Vault - 0.6.x to 0.7.x","doc":"Create a vault module for your project. (Or more than one, if you want!)\n\n    defmodule MyApp.Vault do\n      use Cloak.Vault, otp_app: :my_app\n    end\n\nYou will need to move your existing configuration to the vault. For example,\nif you had this configuration:\n\n    config :cloak, Cloak.AES.CTR,\n      tag: \"AES\",\n      default: true,\n      keys: [\n        %{tag: <<1>>, key: :base64.decode(\"...\"), default: true}\n      ]\n\nYou would convert it to the following:\n\n    config :my_app, MyApp.Vault,\n      ciphers: [\n        default: {Cloak.Ciphers.AES.CTR, tag: \"AES.V2\", key: Base.decode64!(\"...\")},\n        retired: {Cloak.Ciphers.Deprecated.AES.CTR, module_tag: \"AES\", tag: <<1>>, key: Base.decode64!(\"...\")}\n      ]\n\nNotice that the `tag: \"AES\"` became `module_tag: \"AES\"` in the `:retired`\ncipher configuration.\n\nAlternatively, if your keys are stored in environment variables, you could\nconfigure the vault using the `init/1` callback:\n\n    defmodule MyApp.Vault do\n      use Cloak.Vault, otp_app: :my_app\n\n      @impl Cloak.Vault\n      def init(config) do\n        config =\n          Keyword.put(config, :ciphers, [\n            default: {Cloak.Ciphers.AES.CTR, tag: \"AES.V2\", key: decode_env(\"CLOAK_KEY\")},\n            retired: {Cloak.Ciphers.Deprecated.AES.CTR, module_tag: \"AES\", tag: <<1>>, key: decode_env(\"CLOAK_KEY\")}\n          ])\n\n        {:ok, config}\n      end\n\n      defp decode_env(var) do\n        var\n        |> System.get_env()\n        |> Base.decode64!()\n      end\n    end","ref":"0-6-x_to_0-7-x.html#create-a-vault"},{"type":"extras","title":"Create Project-Specific Ecto Types - 0.6.x to 0.7.x","doc":"For each type of encrypted field you have, define a local type. For example,\nif you had the following schema:\n\n    defmodule MyApp.Accounts.User do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      schema \"users\" do\n        field :name, Cloak.EncryptedBinaryField,\n        field :encryption_version\n      end\n\n      @doc false\n      def changeset(struct, attrs \\\\ %{}) do\n        struct\n        |> cast(attrs, [:name])\n        |> put_change(:encryption_version, Cloak.version())\n      end\n    end\n\nYou would define a project-specific field:\n\n    defmodule MyApp.Encrypted.Binary do\n      use Cloak.Fields.Binary, vault: MyApp.Vault\n    end\n\nAnd then replace `Cloak.EncryptedBinaryField` in your schema:\n\n    schema \"users\" do\n      field :name, MyApp.Encrypted.Binary,\n      field :encryption_version\n    end\n\nFinally, you'd remove the `:encryption_version` field as it is no longer\nneeded.\n\n    # In migration...\n    alter table(:users) do\n      remove :encryption_version\n    end\n\n    # In your changeset...\n    @doc false\n    def changeset(struct, attrs \\\\ %{}) do\n      struct\n      |> cast(attrs, [:name])\n    end","ref":"0-6-x_to_0-7-x.html#create-project-specific-ecto-types"},{"type":"extras","title":"Migrate Existing Data - 0.6.x to 0.7.x","doc":"To convert ciphertext en masse from the old `v0.6` format to the new `v0.7`\nformat, you'll need to run `mix cloak.migrate` as shown in its documentation.\n\n    mix cloak.migrate -r MyApp.Repo -s MyApp.Schema","ref":"0-6-x_to_0-7-x.html#migrate-existing-data"},{"type":"extras","title":"Remove `:retired` Cipher - 0.6.x to 0.7.x","doc":"Now that the data has been migrated to the new `v0.7` format, you can remove the\n`:retired` cipher from your configuration.\n\n    config :my_app, MyApp.Vault,\n      ciphers: [\n        default: {Cloak.Ciphers.AES.CTR, tag: \"AES.V2\", key: Base.decode64!(\"...\")}\n      ]","ref":"0-6-x_to_0-7-x.html#remove-retired-cipher"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}